/* GormNibModule.m
 *
 * Copyright (C) 2007 Free Software Foundation, Inc.
 *
 * Author:	Gregory John Casamento <greg_casamento@yahoo.com>
 * Date:	2007
 * 
 * This file is part of GNUstep.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 */

#include <Foundation/Foundation.h>

#include <GormCore/GormCore.h>

#include "GormNibWrapperLoader.h"

@interface GormNibPlugin : GormPlugin
@end

@implementation GormNibPlugin
- (void) didLoad
{
  [self registerDocumentTypeName: [GormNibWrapperLoader fileType]
	humanReadableName: @"Cocoa Nib"
	forExtensions: [NSArray arrayWithObjects: @"nib",nil]];
}
@end

